@@ -21,6 +21,9 @@ class Knode(object):
         self.name = name
         self.kwargs = kwargs
         self.subj = subj
+        if isinstance(col_name, basestring):
+            col_name = [col_name]
+
         self.col_name = col_name
         self.nodes = OrderedDict()
         self.hidden = hidden
@@ -113,7 +116,7 @@ class Knode(object):
 
             #get value for observed node
             if self.observed:
-                kwargs['value'] = grouped_data[self.col_name].values
+                kwargs['value'] = grouped_data[self.col_name] #.to_records(index=False)
 
             # Deterministic nodes require a parent argument that is a
             # dict mapping parent names to parent nodes. Knode wraps
@@ -256,12 +259,13 @@ class Hierarchical(object):
     """
 
     def __init__(self, data, is_group_model=None, depends_on=None, trace_subjs=True,
-                 plot_subjs=False, plot_var=False):
+                 plot_subjs=False, plot_var=False, group_only_nodes=()):
         # Init
         self.plot_subjs = plot_subjs
         self.depends_on = depends_on
         self.mc = None
         self.data = pd.DataFrame(data)
+        self.group_only_nodes = group_only_nodes
 
         if not depends_on:
             depends_on = {}
@@ -507,8 +511,9 @@ class Hierarchical(object):
 
         return self.mc
 
+    @property
     def dic_info(self):
-        """returns information about the model DIC"""
+        """returns information about the model DIC."""
 
         info = {}
         try:
@@ -522,12 +527,16 @@ class Hierarchical(object):
 
         return info
 
+    @property
+    def dic(self):
+        return self.dic_info['DIC']
+
     def _output_stats(self, stats_str, fname=None):
         """
         used by print_stats and print_group_stats to print the stats to the screen
         or to file
         """
-        info = self.dic_info()
+        info = self.dic_info
         if fname is None:
             print stats_str
             print "DIC: %f" % info['DIC']
@@ -730,6 +739,21 @@ class Hierarchical(object):
         for node in nodes.iterrows():
             yield node
 
+    def get_group_traces(self):
+        """Returns a DataFrame containing traces of all stochastic
+        group nodes in the model.
+        """
+        return pd.DataFrame({i.__name__: i.trace() for i in self.get_group_nodes().node})
+
+    def get_traces(self):
+        """Returns a DataFrame containing traces of all stochastic
+        nodes in the model.
+
+        :Note: It is quite easy to then save this trace to csv by
+        calling model.get_traces().to_csv('samples.csv')
+        """
+        return pd.DataFrame({i.__name__: i.trace() for i in self.get_stochastics().node})
+
     @property
     def values(self):
         return {name: node['node'].value[()] for (name, node) in self.iter_non_observeds()}
@@ -752,14 +776,14 @@ class Hierarchical(object):
         else:
             self.map()
 
-    def _partial_optimize(self, optimize_nodes, evaluate_nodes):
+    def _partial_optimize(self, optimize_nodes, evaluate_nodes, fall_to_simplex):
         """Optimize part of the model.
 
         :Arguments:
             nodes : iterable
                 list nodes to optimize.
         """
-        non_observeds = [node for node in optimize_nodes if not node.observed]
+        non_observeds = filter(lambda x: x.observed, optimize_nodes)
 
         init_vals = [node.value for node in non_observeds]
 
@@ -774,14 +798,22 @@ class Hierarchical(object):
             except pm.ZeroProbability:
                 return np.inf
 
+        #optimize
         try:
             fmin_powell(opt, init_vals)
-        except Exception:
-            print "Warning: Powell optimization failed. Falling back to simplex."
-            fmin(opt, init_vals)
+        except Exception as e:
+            if fall_to_simplex:
+                print "Warning: Powell optimization failed. Falling back to simplex."
+                fmin(opt, init_vals)
+            else:
+                raise e
 
-    def approximate_map(self):
+
+    def approximate_map(self, fall_to_simplex = True):
         """Set model to its approximate MAP.
+        Input:
+            fall_to_simplex <bool>
+                should map try using simplex algorithm if powell method failes
         """
         ###############################
         # In order to find the MAP of a hierarchical model one needs
@@ -796,7 +828,7 @@ class Hierarchical(object):
 
         for i in range(len(generations)-1, 0, -1):
             # Optimize the generation at i-1 evaluated over the generation at i
-            self._partial_optimize(generations[i-1], generations[i])
+            self._partial_optimize(generations[i-1], generations[i], fall_to_simplex)
 
         #update map in nodes_db
         self.nodes_db['map'] = np.NaN
@@ -806,228 +838,3 @@ class Hierarchical(object):
             # Some values can be series which we'll just ignore
             except (AttributeError, ValueError):
                 pass
-
-    def create_family_normal(self, name, value=0, g_mu=None,
-                             g_tau=15**-2, var_lower=1e-10,
-                             var_upper=100, var_value=.1):
-        """Create a family of knodes. A family is a group of knodes
-        that belong together.
-
-        For example, a family could consist of the following distributions:
-        * group mean g_mean (Normal(g_mu, g_tau))
-        * group variability g_var (Uniform(var_lower, var_upper))
-        * transform node g_var_trans for g_var (x -> x**-2)
-        * subject (Normal(g_mean, g_var_trans))
-
-        In fact, if is_group_model is True and the name does not appear in
-        group_only nodes, this is the family that will be created.
-
-        Otherwise, only a Normal knode will be returned.
-
-        :Arguments:
-            name : str
-                Name of the family. Each family member will have this name prefixed.
-
-        :Optional:
-            value : float
-                Starting value.
-            g_mu, g_tau, var_lower, var_upper, var_value : float
-                The hyper parameters for the different family members (see above).
-
-        :Returns:
-            OrderedDict: member name -> member Knode
-        """
-        if g_mu is None:
-            g_mu = value
-
-        knodes = OrderedDict()
-
-        if self.is_group_model and name not in self.group_only_nodes:
-            g = Knode(pm.Normal, '%s' % name, mu=g_mu, tau=g_tau,
-                      value=value, depends=self.depends[name])
-            var = Knode(pm.Uniform, '%s_var' % name, lower=var_lower,
-                        upper=var_upper, value=var_value)
-            tau = Knode(pm.Deterministic, '%s_tau' % name,
-                        doc='%s_tau' % name, eval=lambda x: x**-2, x=var,
-                        plot=False, trace=False, hidden=True)
-            subj = Knode(pm.Normal, '%s_subj' % name, mu=g, tau=tau,
-                         value=value, depends=('subj_idx',),
-                         subj=True, plot=self.plot_subjs)
-            knodes['%s'%name] = g
-            knodes['%s_var'%name] = var
-            knodes['%s_tau'%name] = tau
-            knodes['%s_bottom'%name] = subj
-
-        else:
-            subj = Knode(pm.Normal, name, mu=g_mu, tau=g_tau,
-                         value=value, depends=self.depends[name])
-
-            knodes['%s_bottom'%name] = subj
-
-        return knodes
-
-
-    def create_family_trunc_normal(self, name, value=0, lower=None,
-                                   upper=None, var_lower=1e-10,
-                                   var_upper=100, var_value=.1):
-        """Similar to create_family_normal() but creates a Uniform
-        group distribution and a truncated subject distribution.
-
-        See create_family_normal() help for more information.
-
-        """
-        knodes = OrderedDict()
-
-        if self.is_group_model and name not in self.group_only_nodes:
-            g = Knode(pm.Uniform, '%s' % name, lower=lower,
-                      upper=upper, value=value, depends=self.depends[name])
-            var = Knode(pm.Uniform, '%s_var' % name, lower=var_lower,
-                        upper=var_upper, value=var_value)
-            tau = Knode(pm.Deterministic, '%s_tau' % name,
-                        doc='%s_tau' % name, eval=lambda x: x**-2, x=var,
-                        plot=False, trace=False, hidden=True)
-            subj = Knode(pm.TruncatedNormal, '%s_subj' % name, mu=g,
-                         tau=tau, a=lower, b=upper, value=value,
-                         depends=('subj_idx',), subj=True, plot=self.plot_subjs)
-
-            knodes['%s'%name] = g
-            knodes['%s_var'%name] = var
-            knodes['%s_tau'%name] = tau
-            knodes['%s_bottom'%name] = subj
-
-        else:
-            subj = Knode(pm.Uniform, name, lower=lower,
-                         upper=upper, value=value,
-                         depends=self.depends[name])
-            knodes['%s_bottom'%name] = subj
-
-        return knodes
-
-
-    def create_family_invlogit(self, name, value, g_mu=None, g_tau=15**-2,
-                               var_lower=1e-10, var_upper=100, var_value=.1):
-        """Similar to create_family_normal() but adds a invlogit
-        transform knode to the subject and group mean nodes. This is useful
-        when the parameter space is restricted from [0, 1].
-
-        See create_family_normal() help for more information.
-
-        """
-
-        if g_mu is None:
-            g_mu = value
-
-        # logit transform values
-        value_trans = np.log(value) - np.log(1-value)
-        g_mu_trans = np.log(g_mu) - np.log(1-g_mu)
-
-        knodes = OrderedDict()
-
-        if self.is_group_model and name not in self.group_only_nodes:
-            g_trans = Knode(pm.Normal,
-                            '%s_trans'%name,
-                            mu=g_mu_trans,
-                            tau=g_tau,
-                            value=value_trans,
-                            depends=self.depends[name],
-                            plot=False,
-                            hidden=True
-            )
-
-            g = Knode(pm.InvLogit, name, ltheta=g_trans, plot=True,
-                      trace=True)
-
-            var = Knode(pm.Uniform, '%s_var'%name, lower=var_lower,
-                        upper=var_upper, value=var_value)
-
-            tau = Knode(pm.Deterministic, '%s_tau'%name, doc='%s_tau'
-                        % name, eval=lambda x: x**-2, x=var,
-                        plot=False, trace=False, hidden=True)
-
-            subj_trans = Knode(pm.Normal, '%s_subj_trans'%name,
-                               mu=g_trans, tau=tau, value=value_trans,
-                               depends=('subj_idx',), subj=True,
-                               plot=False, hidden=True)
-
-            subj = Knode(pm.InvLogit, '%s_subj'%name,
-                         ltheta=subj_trans, depends=('subj_idx',),
-                         plot=self.plot_subjs, trace=True, subj=True)
-
-            knodes['%s_trans'%name]      = g_trans
-            knodes['%s'%name]            = g
-            knodes['%s_var'%name]        = var
-            knodes['%s_tau'%name]        = tau
-
-            knodes['%s_subj_trans'%name] = subj_trans
-            knodes['%s_bottom'%name]     = subj
-
-        else:
-            g_trans = Knode(pm.Normal, '%s_trans'%name, mu=g_mu,
-                            tau=g_tau, value=value_trans,
-                            depends=self.depends[name], plot=False, hidden=True)
-
-            g = Knode(pm.InvLogit, '%s'%name, ltheta=g_trans, plot=True,
-                      trace=True )
-
-            knodes['%s_trans'%name] = g_trans
-            knodes['%s_bottom'%name] = g
-
-        return knodes
-
-    def create_family_exp(self, name, value=0, g_mu=None,
-                          g_tau=15**-2, var_lower=1e-10, var_upper=100, var_value=.1):
-        """Similar to create_family_normal() but adds an exponential
-        transform knode to the subject and group mean nodes. This is useful
-        when the parameter space is restricted from [0, +oo).
-
-        See create_family_normal() help for more information.
-
-        """
-        if g_mu is None:
-            g_mu = value
-
-        value_trans = np.log(value)
-        g_mu_trans = np.log(g_mu)
-
-        knodes = OrderedDict()
-        if self.is_group_model and name not in self.group_only_nodes:
-            g_trans = Knode(pm.Normal, '%s_trans' % name, mu=g_mu_trans,
-                            tau=g_tau, value=value_trans,
-                            depends=self.depends[name], plot=False, hidden=True)
-
-            g = Knode(pm.Deterministic, '%s'%name, eval=lambda x: np.exp(x),
-                      x=g_trans, plot=True)
-
-            var = Knode(pm.Uniform, '%s_var' % name,
-                        lower=var_lower, upper=var_upper, value=var_value)
-
-            tau = Knode(pm.Deterministic, '%s_tau' % name, eval=lambda x: x**-2,
-                        x=var, plot=False, trace=False, hidden=True)
-
-            subj_trans = Knode(pm.Normal, '%s_subj_trans'%name, mu=g_trans,
-                         tau=tau, value=value_trans, depends=('subj_idx',),
-                         subj=True, plot=False, hidden=True)
-
-            subj = Knode(pm.Deterministic, '%s_subj'%name, eval=lambda x: np.exp(x),
-                         x=subj_trans,
-                         depends=('subj_idx',), plot=self.plot_subjs,
-                         trace=True, subj=True)
-
-            knodes['%s_trans'%name]      = g_trans
-            knodes['%s'%name]            = g
-            knodes['%s_var'%name]        = var
-            knodes['%s_tau'%name]        = tau
-            knodes['%s_subj_trans'%name] = subj_trans
-            knodes['%s_bottom'%name]     = subj
-
-        else:
-            g_trans = Knode(pm.Normal, '%s_trans' % name, mu=g_mu_trans,
-                            tau=g_tau, value=value_trans,
-                            depends=self.depends[name], plot=False, hidden=True)
-
-            g = Knode(pm.Deterministic, '%s'%name, doc='%s'%name, eval=lambda x: np.exp(x), x=g_trans, plot=True)
-            knodes['%s_trans'%name] = g_trans
-            knodes['%s_bottom'%name] = g
-
-        return knodes
-
